import { ResourceItemType } from "@/Common/ResourceItem/ResourceItemType";
import { NetRequest } from "../../Engine/CatanEngine/NetManagerV2/NetRequest";

export namespace TxnRequest {
	/**贈禮資訊 */
	export type TxnInfo = [
		serialNum: number,
		time: number,
		giver: [aId: number, nickName: string],
		receiver: [aId: number, nickName: string],
		status: string,
		fee: number,
		goodsInfo: GoodsInfo[],
	];
	/**物品資訊 */
	export type GoodsInfo = [
		itemType: ResourceItemType,
		id: number,
		count: number
	];
	/**凍結資訊 */
	export type FrozenInfo = [
		time: number,
		type: StoredValueType,
		amount: number,
	];
	/**儲值類型(凍結類別) */
	export enum StoredValueType {
		StoredValue = 1, // wait for notifications on web functions
		StoredValueBonus,
		RewardsSharing,
		PackageSerialNumber,
		BindingRewards,
		InitialGoldCoins,
		FortuneGold,
		Family,
		Backpack,
		LineFriend
	}
	/**贈禮狀態 */
	export enum Status {
		Applying = 0,
		GiverGift = 1,
		Authentication = 2,
		Receive = 3,
		Cancel = 11,
		Reject = 12,
		Timeout = 13,
		Done = 21,
		CancelComplete = 22,
		RefuseComplete = 23,
		TimeoutComplete = 24,
		Unknown = 25,
	}
	/**常用名單玩家資訊 */
	export type PlayerInfo = [
		aId: number,
		nickName: string,
		vip: number,
		/**上線狀態(0: 離線, 1: 在線) */
		onlineStatus: 0 | 1,
	];
	/**個人贈禮可使用額度 */
	export interface UsageQuotaSelf {
		/**可送數量 */
		msm: number;
		/**可送額度 */
		sm: number;
		/**可送次數 */
		sc: number;
		/**凍結總額 */
		fm: number;
	}
	// =======================================================================================
	/**贈禮中心 */
	export class Center extends NetRequest<null, TxnInfo[]> {
		get Method(): string {
			return "txn.center";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	/**個人贈禮資訊 */
	export class GetInfoSelf extends NetRequest<null, UsageQuotaSelf> {
		get Method(): string {
			return "txn.info";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	enum GetFrozenInfoError {
		PhoneNotBound = 11,
		GiftGivingPunishment,
		GettingGiftInfoError,
		StatusError,
		OTPRequestTimeTooShort,
		OTPSendingFailed,
	}
	/**凍結資訊 */
	export class GetFrozenInfo extends NetRequest<null, FrozenInfo[]> {
		get Method(): string {
			return "txn.frozen";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	enum CreateNewError {
		giftGivingPunishment = 11,
		UnboundMobilePhone,
		Unloaded,
		TargetNotExist,
		OnTargetGiftPenalty,
		TargetVIPLevel,
		TargetAIDError,
		AimIsPlayerItsSelf,
		ExceededNumbersOfGifts,
		GiftItems_resource,
		GiftItems_json_resource,
		MoneyMistakes,
		InsufficientAmount,
		ExceedingAvailableGiftAmount,
		BuildFailed,
		FailedGettingTargetCache,
	}
	interface CreateNewRequest {
		a?: number;
		n?: string;
		i: GoodsInfo[];
		f?: CreateNewFlagType;
	}
	export enum CreateNewFlagType {
		/**新增常用名單 */
		AddFrequentlyUsedList,
	}
	/**建立贈禮 */
	export class CreateNew extends NetRequest<CreateNewRequest, TxnInfo> {
		get Method(): string {
			return "txn.new";
		}
		constructor(name: string, i: GoodsInfo[], flag?: CreateNewFlagType) {
			super();
			this.Data = {
				n: name,
				i: i,
				f: flag
			};
		}
	}
	// =======================================================================================
	interface TradeRequest {
		/**流水號 */
		id: number;
		/**操作類型 */
		t: TradeOperationType;
		/**密碼 */
		c?: string;
	}
	/**贈禮操作類型 */
	export enum TradeOperationType {
		/**同意(收禮方) */
		Agree_Receiver = 1,
		/**認證(贈禮方) */
		Authentication_Giver,
		/**取得(收禮方) */
		Obtain_Receiver,
		/**取回(贈禮方) */
		Retrieve_Giver,
		/**取消(贈禮方) */
		Cancellation_Giver,
		/**拒絕(收禮方) */
		Reject_Receiver,
	}
	export interface TradeResponse {
		id: number;
		s: Status;
		p: GoodsInfo[];
	}
	export enum TradeError {
		TargetGiftInPenalty = 11,
		GiftInPenalty,
		GiftNotExist,
		OperationError,
		WrongPassword,
	}
	/**贈禮操作 */
	export class Trade extends NetRequest<TradeRequest, TradeResponse> {
		get Method(): string {
			return "txn.trade";
		}
		constructor(txn: number, type: TradeOperationType, c?: string) {
			super();
			this.Data = {
				id: txn,
				t: type,
				c: c,
			};
		}
	}
	// =======================================================================================
	/**贈禮常用名單 */
	export class GetUserList extends NetRequest<null, PlayerInfo[]> {
		get Method(): string {
			return "txn.user_list";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	export type UserAddResponse = {
		u: PlayerInfo;
		/*
		 * 移除的玩家AID
		 * */
		r?: number;
	};
	export enum UserAddError {
		TargetIsUserItsSelf = 11,
		TargetHasAlreadyBeenExist,
		VIPLevel,
		TargetNotExist,
	}
	/**增加常用名單 */
	export class UserAdd extends NetRequest<{ n: string }, UserAddResponse> {
		get Method(): string {
			return "txn.user_add";
		}
		constructor(nickName: string) {
			super();
			this.Data = {
				n: nickName
			};
		}
	}
	// =======================================================================================
	export enum UserDelError {
		TargetIsUserItsSelf = 11,
		TargetHasNotExist,
	}
	/**移除常用名單 */
	export class UserDel extends NetRequest<{ a: number }, null> {
		get Method(): string {
			return "txn.user_del";
		}
		constructor(aid: number) {
			super();
			this.Data = {
				a: aid
			};
		}
	}
	// =======================================================================================
	interface AuthCodeResquest {
		id: number
	}
	export interface AuthCodeResponse {
		t: number;
		c: string;
	}
	export enum CodeError {
		PhoneNotBound = 11,
		GiftGivingPunishment,
		GettingGiftInfoError,
		StatusError,
		OTPRequestTimeTooShort,
		OTPSendingFailed,
	}
	/**請SERVER發送贈禮的簡訊OPT認證碼(未設定贈禮密碼) */
	export class AuthCode extends NetRequest<AuthCodeResquest, AuthCodeResponse> {
		get Method(): string {
			return "txn.code";
		}
		constructor(txn: number) {
			super();
			this.Data = {
				id: txn,
			};
		}
	}
	// =======================================================================================
	export enum SetPassWordError {
		PasswordHasBeenSet = 11,
		PhoneNotBound,
		OTPError,
	}
	/**設定贈禮密碼(簡訊OTP驗證建立新密碼) */
	export class SetPassWord extends NetRequest<{ pw: string; c: string; }, null> {
		get Method(): string {
			return "txn.pw";
		}
		constructor(pw: string, c: string) {
			super();
			this.Data = {
				pw: pw,
				c: c
			};
		}
	}
	// =======================================================================================
	enum ChangePassWordError {
		PasswordNotSet = 11,
		PhoneNotBound,
		WrongPasswordFormat,
		PasswordAuthenticationFailed,
	}
	/**變更贈禮密碼 */
	export class ChangePassWord extends NetRequest<{ opw: string; pw: string; }, null> {
		get Method(): string {
			return "txn.pw_change";
		}
		constructor(opw: string, pw: string) {
			super();
			this.Data = {
				opw: opw,
				pw: pw
			};
		}
	}
	// =======================================================================================
	export enum GiftForgotError {
		PasswordHasNotBeenSetYet = 11,
		PhoneNotBound,
		OTPError,
	}
	/**忘記密碼(簡訊OTP驗證建立新密碼) */
	export class GiftForgot extends NetRequest<{ pw: string; c: string; }, null> {
		get Method(): string {
			return "txn.pw_fg";
		}
		constructor(pw: string, c: string) {
			super();
			this.Data = {
				pw: pw,
				c: c
			};
		}
	}

	// =======================================================================================
	interface GetPassWordResponse {
		t: number;
		c: string;
	}
	export enum GetPassWordError {
		PasswordHasBeenSet = 11,
		PhoneNotBound,
		OTPRequestTimeTooShort,
		OTPSendingFailed,
	}
	/**取得贈禮密碼OTP */
	export class GetPassWord extends NetRequest<null, GetPassWordResponse> {
		get Method(): string {
			return "txn.pw_code";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	interface GetForgotPassWordResponse {
		t: number;
		c: string;
	}
	export enum GetForgotPassWordError {
		PasswordHasNotBeenSetYet = 11,
		PhoneNotBound,
		OTPRequestTimeTooShort,
		OTPSendingFailed,
	}
	/**取得贈禮忘記密碼OTP */
	export class GetForgotPassWord extends NetRequest<null, GetForgotPassWordResponse> {
		get Method(): string {
			return "txn.pw_fg_code";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	/**贈禮紀錄 */
	export class Record extends NetRequest<null, TxnInfo[]> {
		get Method(): string {
			return "txn.record";
		}
		constructor() {
			super();
		}
	}
	// =======================================================================================
	enum GetListError {
		NotExist = 11,
	}
	/**取得贈禮品項 */
	export class GetTxnList extends NetRequest<{ t: number[] }, TxnInfo[]> {
		get Method(): string {
			return "txn.get";
		}
		constructor(txn: number[]) {
			super();
			this.Data = {
				t: txn,
			};
		}
	}

	// =======================================================================================
}